{
  This demo project shows how to use regular expressions for searching in
  TRichView.

  This is a simple version of this demo: searching from cursor

  This demo requires Delphi XE or newer.

  https://www.trichview.com

}


unit MainFrm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, RegularExpressions,
  Controls, Forms, Dialogs, Buttons, ExtCtrls, RVScroll,
  RichView, RVEdit, StdCtrls, RVStyle;

type
  TfrmMain = class(TForm)
    rve: TRichViewEdit;
    Panel1: TPanel;
    btnOpen: TSpeedButton;
    btnFind: TSpeedButton;
    Label1: TLabel;
    txtRegEx: TEdit;
    cbIgnoreCase: TCheckBox;
    RVStyle1: TRVStyle;
    OpenDialog1: TOpenDialog;
    procedure FormCreate(Sender: TObject);
    procedure rveChange(Sender: TObject);
    procedure btnFindClick(Sender: TObject);
    procedure txtRegExChange(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure cbIgnoreCaseClick(Sender: TObject);
    procedure btnOpenClick(Sender: TObject);
  private
    { Private declarations }
    FTextChanged, FExpressionChanged: Boolean;
    FText: String;
    FPRegEx: ^TRegEx;
    procedure UpdateText;
    procedure Select(const M: TMatch);
    function GetSearchOptions: TRegExOptions;
  public
    { Public declarations }
    procedure Search(StartPos: Integer);
  end;

var
  frmMain: TfrmMain;

implementation

uses
  RVLinear, CRVData, CRVFData;

{$R *.dfm}

{ TfrmMain }


procedure TfrmMain.FormCreate(Sender: TObject);
begin
  FTextChanged := True;
  FExpressionChanged := True;
end;

procedure TfrmMain.FormDestroy(Sender: TObject);
begin
  if FPRegEx <> nil then
    Dispose(FPRegEx);
end;

procedure TfrmMain.btnOpenClick(Sender: TObject);
begin
  if OpenDialog1.Execute then
  begin
    rve.Clear;
    rve.DeleteUnusedStyles(True, True, True);
    case OpenDialog1.FilterIndex of
      1: rve.LoadRVF(OpenDialog1.FileName);
      2: rve.LoadRTF(OpenDialog1.FileName);
      3: rve.LoadText(OpenDialog1.FileName, 0, 0, False);
      4: rve.LoadTextW(OpenDialog1.FileName, 0, 0, False);
    end;
    rve.Format;
    rveChange(Sender);
  end;
end;

procedure TfrmMain.rveChange(Sender: TObject);
begin
  FTextChanged := True;
  txtRegExChange(Sender);
end;

procedure TfrmMain.txtRegExChange(Sender: TObject);
begin
  FExpressionChanged := True;
end;

procedure TfrmMain.cbIgnoreCaseClick(Sender: TObject);
begin
  txtRegExChange(Sender);
end;

procedure TfrmMain.Select(const M: TMatch);
var
  StartIndex, EndIndex, ItemNo1, Offs1, ItemNo2, Offs2: Integer;
  RVData1, RVData2: TCustomRVData;
begin
  StartIndex := M.Index - 1;
  EndIndex := StartIndex + M.Length;
  if LinearToRichView(rve, rve.RVData, StartIndex, RVData1, ItemNo1, Offs1) and
     LinearToRichView(rve, rve.RVData, EndIndex, RVData2, ItemNo2, Offs2) and
     (RVData1 = RVData2) then
  begin
    RVData1 := RVData1.Edit;
    TCustomRVFormattedData(RVData1).SetSelectionBounds(ItemNo1, Offs1, ItemNo2, Offs2);
    rve.RefreshAll;
  end
  else
    Application.MessageBox('The matched fragment cannot be selected', 'Error', MB_OK);
end;

procedure TfrmMain.Search(StartPos: Integer);
var
  M: TMatch;
begin
  if FExpressionChanged then
  begin
    if FPRegEx <> nil then
      Dispose(FPRegEx);
    New(FPRegEx);
    FPRegEx^.Create(txtRegEx.Text, GetSearchOptions);
  end;
  M := FPRegEx^.Match(FText, StartPos);
  if (M.Index < 1) or (M.Length = 0) then
  begin
    Application.MessageBox('The expression does not match', 'End of Search', MB_OK);
    exit;
  end;
  Select(M);
end;


procedure TfrmMain.btnFindClick(Sender: TObject);
var
  StartPos: Integer;
begin
  if txtRegEx.Text = '' then
  begin
    Application.MessageBox('Please enter an expression to search', 'Empty Expression', MB_OK);
    txtRegEx.SetFocus;
    exit;
  end;
  rve.SetFocus;
  UpdateText;
  StartPos := RVGetLinearCaretPos(rve) + 1;
  if Length(FText) < StartPos then
  begin
    Application.MessageBox('End of text', 'End of Search', MB_OK);
    exit;
  end;
  Search(StartPos);
  FExpressionChanged := False;
end;


procedure TfrmMain.UpdateText;
begin
  if FTextChanged then
  begin
    FText := RVGetTextRange(rve, 0, RVGetTextLength(rve));
    FTextChanged := False;
  end;
end;

function TfrmMain.GetSearchOptions: TRegExOptions;
begin
  Result := [roMultiLine];
  if cbIgnoreCase.Checked then
    Include(Result, roIgnoreCase);
end;

end.
