unit ColorBlendMarks;

interface

uses
  Classes, Windows, Graphics, RVWordPaint, DLines, RVItem, CRVData, RichView,
  StrUtils, SysUtils, RVTypes;

type

  TRVMarkTextPainter = class (TRVWordPainter)
  private
    FColor: TColor;
    FOpacity: Byte;
    procedure EnumItemsClear(RVData: TCustomRVData; ItemNo: Integer;
      var UserData1: Integer; const UserData2: TRVUnicodeString;
      var ContinueEnum: Boolean);
    procedure EnumItemsAdd(RVData: TCustomRVData; ItemNo: Integer;
      var UserData1: Integer; const UserData2: TRVUnicodeString;
      var ContinueEnum: Boolean);
  public
    constructor Create(AStartOffs, ALength: Integer; AColor: TColor;
      AOpacity: Byte);
    procedure Draw(Canvas: TCanvas; ditem: TRVDrawLineInfo;
      RVData: TPersistent; const r: TRect; Index: Integer); override;
  end;

procedure AddRectMark(RVData: TCustomRVData; ItemNo, Offs, Len: Integer;
  Color: TColor; Opacity: Byte = 100);
procedure MarkSubstring(rv: TCustomRichView; const s: String; Color: TColor);
procedure ClearRectMarks(rv: TCustomRichView);

implementation

constructor TRVMarkTextPainter.Create(AStartOffs, ALength: Integer;
  AColor: TColor; AOpacity: Byte);
begin
  inherited Create(AStartOffs, ALength);
  FColor := AColor;
  FOpacity := AOpacity;
end;

procedure TRVMarkTextPainter.Draw(Canvas: TCanvas; ditem: TRVDrawLineInfo;
  RVData: TPersistent; const r: TRect; Index: Integer);
begin
  TCustomRVData(RVData).GetRVStyle.GraphicInterface.FillRectAlpha(Canvas, r, FColor, FOpacity);
end;

procedure TRVMarkTextPainter.EnumItemsClear(RVData: TCustomRVData; ItemNo: Integer;
  var UserData1: Integer; const UserData2: TRVUnicodeString; var ContinueEnum: Boolean);
var i: Integer;
    WordPaintList: TRVWordPainterList;
begin
  WordPaintList := RVData.GetItem(ItemNo).WordPaintList;
  if RVData.GetItem(ItemNo).WordPaintList<>nil then
  begin
    for i := WordPaintList.Count - 1 downto 0 do
      if WordPaintList[i] is TRVMarkTextPainter then
        WordPaintList.Delete(i);
    if RVData.GetItem(ItemNo).WordPaintList=nil then
    begin
      RVData.GetItem(ItemNo).WordPaintList.Free;
      RVData.GetItem(ItemNo).WordPaintList := nil;
    end;
  end;
  ContinueEnum := True;
end;

procedure TRVMarkTextPainter.EnumItemsAdd(RVData: TCustomRVData; ItemNo: Integer;
  var UserData1: Integer; const UserData2: TRVUnicodeString; var ContinueEnum: Boolean);
var s: TRVUnicodeString;
    pc, pctofind, pcfound: PRVUnicodeChar;
begin
  if RVData.GetItemStyle(ItemNo)<0 then
    exit;
  s := RVData.GetItemTextW(ItemNo);
  pc := PRVUnicodeChar(s);
  pctofind := PRVUnicodeChar(UserData2);
  pcfound := StrPos(pc, pctofind);
  while pcfound<>nil do begin
    AddRectMark(RVData, ItemNo, pcfound-pc+1, System.Length(UserData2), TColor(UserData1));
    pcfound := StrPos(pcfound+System.Length(UserData2), pctofind);
  end;
  ContinueEnum := True;
end;

procedure AddRectMark(RVData: TCustomRVData; ItemNo, Offs, Len: Integer;
  Color: TColor; Opacity: Byte);
var Item: TCustomRVItemInfo;
begin
  Item := RVData.GetItem(ItemNo);
  if Item.StyleNo<0 then
    exit;
  if Item.WordPaintList=nil then
    Item.WordPaintList := TRVWordPainterList.Create;
  Item.WordPaintList.Add(TRVMarkTextPainter.Create(Offs, Len, Color, Opacity))
end;

procedure MarkSubstring(rv: TCustomRichView; const s: String; Color: TColor);
var FakePainter: TRVMarkTextPainter;
var v: Integer;
begin
  FakePainter := nil;
  v := Integer(Color);
  rv.RVData.EnumItems(FakePainter.EnumItemsAdd, v, s);
end;

procedure ClearRectMarks(rv: TCustomRichView);
var FakePainter: TRVMarkTextPainter;
var v: Integer;
begin
  FakePainter := nil;
  v := 0;
  rv.RVData.EnumItems(FakePainter.EnumItemsClear, v, '');
end;

end.
