{
  This demo project shows how to use regular expressions for searching in
  TRichView.

  This is an advanced version of the demo: searching and highlighting all
  all matches.

  This version of demo is not appropriate for too large documents, because
  it uses Matches() method which may be too slow. In a real application,
  you can use Match() and then a cycle of Match.NextMatch, and limit a count of
  results.

  This demo requires Delphi XE or newer.

  https://www.trichview.com

}


unit MainFrm;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, RegularExpressions,
  Controls, Forms, Dialogs, Buttons, ExtCtrls, RVScroll,
  RichView, RVEdit, StdCtrls, RVStyle;

type
  TfrmMain = class(TForm)
    rve: TRichViewEdit;
    Panel1: TPanel;
    btnOpen: TSpeedButton;
    btnFind: TSpeedButton;
    Label1: TLabel;
    txtRegEx: TEdit;
    cbIgnoreCase: TCheckBox;
    RVStyle1: TRVStyle;
    OpenDialog1: TOpenDialog;
    btnNext: TSpeedButton;
    lblResult: TLabel;
    btnPrior: TSpeedButton;
    procedure FormCreate(Sender: TObject);
    procedure rveChange(Sender: TObject);
    procedure btnFindClick(Sender: TObject);
    procedure btnOpenClick(Sender: TObject);
    procedure btnPriorClick(Sender: TObject);
    procedure btnNextClick(Sender: TObject);
  private
    { Private declarations }
    FTextChanged: Boolean;
    FText: String;
    FMatches: TMatchCollection;
    FMatchIndex: Integer;
    procedure UpdateText;
    procedure AddMatch(const M: TMatch);
    function GetSearchOptions: TRegExOptions;
    procedure UpdateUI;
    procedure Select(const M: TMatch);
  public
    { Public declarations }
    procedure Search;
  end;

var
  frmMain: TfrmMain;

implementation

uses
  RVTypes, RVLinear, CRVData, CRVFData, RVItem, ColorBlendMarks;

{$R *.dfm}

{ TfrmMain }


const
  MarkColor = $DDEE;

procedure TfrmMain.FormCreate(Sender: TObject);
begin
  FTextChanged := True;
  FMatchIndex := -1;
  UpdateUI;
end;

procedure TfrmMain.btnOpenClick(Sender: TObject);
begin
  if OpenDialog1.Execute then
  begin
    rve.Clear;
    rve.DeleteUnusedStyles(True, True, True);
    case OpenDialog1.FilterIndex of
      1: rve.LoadRVF(OpenDialog1.FileName);
      2: rve.LoadRTF(OpenDialog1.FileName);
      3: rve.LoadText(OpenDialog1.FileName, 0, 0, False);
      4: rve.LoadTextW(OpenDialog1.FileName, 0, 0, False);
    end;
    rve.Format;
    rveChange(Sender);
  end;
end;

procedure TfrmMain.btnPriorClick(Sender: TObject);
begin
  if FMatchIndex > 0 then
  begin
    dec(FMatchIndex);
    Select(FMatches[FMatchIndex]);
    UpdateUI;
  end;
end;

procedure TfrmMain.btnNextClick(Sender: TObject);
begin
  if (FMatchIndex >= 0) and (FMatchIndex < FMatches.Count - 1) then
  begin
    inc(FMatchIndex);
    Select(FMatches[FMatchIndex]);
    UpdateUI;
  end;
end;

procedure TfrmMain.rveChange(Sender: TObject);
begin
  FTextChanged := True;
  if FMatchIndex >= 0 then
  begin
    ClearRectMarks(rve);
    rve.RefreshAll;
    FMatchIndex := -1;
    UpdateUI;
  end;
end;

procedure TfrmMain.AddMatch(const M: TMatch);
var
  i, StartIndex, EndIndex, ItemNo1, Offs1, ItemNo2, Offs2: Integer;
  RVData1, RVData2: TCustomRVData;
begin
  StartIndex := M.Index - 1;
  EndIndex := StartIndex + M.Length;
  if LinearToRichView(rve, rve.RVData, StartIndex, RVData1, ItemNo1, Offs1) and
     LinearToRichView(rve, rve.RVData, EndIndex, RVData2, ItemNo2, Offs2) and
     (RVData1 = RVData2) then
  begin
    if (ItemNo1 = ItemNo2) and (RVData1.GetItemStyle(ItemNo1)>=0) then
      AddRectMark(RVData1, ItemNo1, Offs1, Offs2 - Offs1, MarkColor)
    else
    begin
      if RVData1.GetItemStyle(ItemNo1)>=0 then
        AddRectMark(RVData1, ItemNo1, Offs1,
          RVData1.GetOffsAfterItem(ItemNo1) - Offs1, MarkColor);
      for i := ItemNo1 + 1 to ItemNo2 - 1 do
        if RVData1.GetItemStyle(i)>=0 then
          AddRectMark(RVData1, i, RVData1.GetOffsBeforeItem(i),
            RVData1.GetOffsAfterItem(i) - RVData1.GetOffsBeforeItem(i), MarkColor);
      if RVData1.GetItemStyle(ItemNo2)>=0 then
        AddRectMark(RVData1, ItemNo2, RVData1.GetOffsBeforeItem(ItemNo2),
          Offs2 - RVData1.GetOffsBeforeItem(ItemNo2), MarkColor);
    end;
  end;
end;

procedure TfrmMain.Search;
var
  i: Integer;
begin
  Screen.Cursor := crHourGlass;
  try
    ClearRectMarks(rve);
    rve.RefreshAll;
    FMatchIndex := -1;
    FMatches := TRegEx.Matches(FText, txtRegEx.Text, GetSearchOptions);
    if FMatches.Count = 0 then
      Application.MessageBox('No matches found', 'End of Search', MB_OK)
    else
    begin
       for i := 0 to FMatches.Count - 1 do
         AddMatch(FMatches[i]);
       rve.RefreshAll;
       FMatchIndex := 0;
       Select(FMatches[0]);
    end;
    UpdateUI;
  finally
    Screen.Cursor := crDefault;
  end;
end;

procedure TfrmMain.Select(const M: TMatch);
var
  StartIndex, EndIndex, ItemNo1, Offs1, ItemNo2, Offs2: Integer;
  RVData1, RVData2: TCustomRVData;
begin
  StartIndex := M.Index - 1;
  EndIndex := StartIndex + M.Length;
  if LinearToRichView(rve, rve.RVData, StartIndex, RVData1, ItemNo1, Offs1) and
     LinearToRichView(rve, rve.RVData, EndIndex, RVData2, ItemNo2, Offs2) and
     (RVData1 = RVData2) then
  begin
    RVData1 := RVData1.Edit;
    TCustomRVFormattedData(RVData1).SetSelectionBounds(ItemNo1, Offs1, ItemNo2, Offs2);
    rve.RefreshAll;
  end
  else
    Application.MessageBox('The matched fragment cannot be selected', 'Error', MB_OK);
end;


procedure TfrmMain.btnFindClick(Sender: TObject);
begin
  if txtRegEx.Text = '' then
  begin
    Application.MessageBox('Please enter an expression to search', 'Empty Expression', MB_OK);
    txtRegEx.SetFocus;
    exit;
  end;
  rve.SetFocus;
  UpdateText;
  Search;
end;


procedure TfrmMain.UpdateText;
begin
  if FTextChanged then
  begin
    FText := RVGetTextRange(rve, 0, RVGetTextLength(rve));
    FTextChanged := False;
  end;
end;

procedure TfrmMain.UpdateUI;
begin
  btnNext.Enabled := (FMatchIndex>=0) and (FMatches.Count > 0) and (FMatchIndex < FMatches.Count - 1);
  btnPrior.Enabled := (FMatchIndex>=0) and (FMatches.Count > 0) and (FMatchIndex > 0);
  if FMatchIndex < 0 then
    lblResult.Caption := 'No active search'
  else
    lblResult.Caption := IntToStr(FMatchIndex + 1) + ' of ' + IntToStr(FMatches.Count);
end;

function TfrmMain.GetSearchOptions: TRegExOptions;
begin
  Result := [roMultiLine];
  if cbIgnoreCase.Checked then
    Include(Result, roIgnoreCase);
end;

end.
